/*===========================================================================*
 * png.c                                                                     *
 *                                                                           *
 *  procedures to deal with PNG files                                        *
 *                                                                           *
 * EXPORTED PROCEDURES                                                       *
 *  png_readpng                                                              *
 *                                                                           *
 *===========================================================================*/


/*  
 *  $Header: /Projects/graphics/mpeg_encode/mpeg_encode/png.c,v 1.1 2000/07/20 00:44:19 dick Exp $
 *  $Log: png.c,v $
 *  Revision 1.1  2000/07/20 00:44:19  dick
 *  PNG interface file for mpeg_encode
 *
 */


/*==============*
 * HEADER FILES *
 *==============*/

#if 0
#include <stdio.h>
#ifndef	_WIN32
	#include <unistd.h> /* lseek */
#else
	#include <io.h>
#endif
#endif
#include "all.h"
#include "mtypes.h"
#include "frames.h"
#include "prototypes.h"
#include "param.h"
#include "readframe.h"
#include "fsize.h"
#include "rgbtoycc.h"
//#include "jpeg.h"
#include "pnglib/png.h"

static void error(text)
char *text;
{
  fprintf(stderr,text);
  putc('\n',stderr);
  exit(1);
}


xel** png_readpng(FILE* fp, int* colsP, int* rowsP, xelval* maxvalP, int* formatP )
{
/* From lpng example.c */
	png_structp png_ptr;
	png_infop info_ptr;
	png_bytep* row_pointers;
	unsigned int sig_read = 0;
	int bit_depth, color_type, interlace_type;
	int i;
//	int r, g, b;

/* Read a PNG file.  You may want to return an error code if the read
 * fails (depending upon the failure).  There are two "prototypes" given
 * here - one where we are given the filename, and we need to open the
 * file, and the other where we are given an open file (possibly with
 * some or all of the magic bytes read - see comments above).
 */


   /* Create and initialize the png_struct with the desired error handler
    * functions.  If you want to use the default stderr and longjump method,
    * you can supply NULL for the last three parameters.  We also supply the
    * the compiler header file version, so that we know if the application
    * was compiled with a compatible version of the library.  REQUIRED
    */
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
	   NULL, NULL, NULL);
//      png_voidp user_error_ptr, user_error_fn, user_warning_fn);

   if (png_ptr == NULL)
     error("Couldn't create png_ptr\n");

   /* Allocate/initialize the memory for image information.  REQUIRED. */
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL)
   {
      fclose(fp);
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
      error("Couldn't create info_ptr\n");
   }

   /* Set error handling if you are using the setjmp/longjmp method (this is
    * the normal method of doing things with libpng).  REQUIRED unless you
    * set up your own error handlers in the png_create_read_struct() earlier.
    */
   if (setjmp(png_ptr->jmpbuf))
   {
      /* Free all of the memory associated with the png_ptr and info_ptr */
      png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
      fclose(fp);
      /* If we get here, we had a problem reading the file */
      return(NULL);
   }

   /* Set up the input control if you are using standard C streams */
   png_init_io(png_ptr, fp);

   /* If we have already read some of the signature */
   png_set_sig_bytes(png_ptr, sig_read);

   /* The call to png_read_info() gives us all of the information from the
    * PNG file before the first IDAT (image data chunk).  REQUIRED
    */
   png_read_info(png_ptr, info_ptr);

   png_get_IHDR(png_ptr, info_ptr, colsP, rowsP, &bit_depth, &color_type,
       &interlace_type, NULL, NULL);

/**** Set up the data transformations you want.  Note that these are all
 **** optional.  Only call them if you want/need them.  Many of the
 **** transformations only work on specific types of images, and many
 **** are mutually exclusive.
 ****/

   /* tell libpng to strip 16 bit/color files down to 8 bits/color */
   png_set_strip_16(png_ptr);

   /* Strip alpha bytes from the input data without combining with the
    * background (not recommended).
    */
   //png_set_strip_alpha(png_ptr);

   /* Extract multiple pixels with bit depths of 1, 2, and 4 from a single
    * byte into separate bytes (useful for paletted and grayscale images).
    */
   png_set_packing(png_ptr);

   /* Change the order of packed pixels to least significant bit first
    * (not useful if you are using png_set_packing). */
   //png_set_packswap(png_ptr);

   /* Expand paletted colors into true RGB triplets */
   if (color_type == PNG_COLOR_TYPE_PALETTE)
      png_set_expand(png_ptr);

   /* Expand grayscale images to the full 8 bits from 1, 2, or 4 bits/pixel */
   //if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
      //png_set_expand(png_ptr);

   /* Expand paletted or RGB images with transparency to full alpha channels
    * so the data will be available as RGBA quartets.
    */
   //if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
      //png_set_expand(png_ptr);

   /* Set the background color to draw transparent and alpha images over.
    * It is possible to set the red, green, and blue components directly
    * for paletted images instead of supplying a palette index.  Note that
    * even if the PNG file supplies a background, you are not required to
    * use it - you should use the (solid) application background if it has one.
    */
#if 0
   png_color_16 my_background, *image_background;

   if (png_get_bKGD(png_ptr, info_ptr, &image_background))
      png_set_background(png_ptr, image_background,
                         PNG_BACKGROUND_GAMMA_FILE, 1, 1.0);
   else
      png_set_background(png_ptr, &my_background,
                         PNG_BACKGROUND_GAMMA_SCREEN, 0, 1.0);
#endif

#if 0
	/* wild guess the gamma.  2.2 is maybe a typical PC */
	screen_gamma = 2.2;

   /* Tell libpng to handle the gamma conversion for you.  The second call
    * is a good guess for PC generated images, but it should be configurable
    * by the user at run time by the user.  It is strongly suggested that
    * your application support gamma correction.
    */

   int intent;

   if (png_get_sRGB(png_ptr, info_ptr, &intent))
      png_set_sRGB(png_ptr, intent, 0);
   else
   {
      double image_gamma;
      if (png_get_gAMA(png_ptr, info_ptr, &image_gamma))
         png_set_gamma(png_ptr, screen_gamma, image_gamma);
      else
         png_set_gamma(png_ptr, screen_gamma, 0.45455);
   }
#endif
   /* Allocate the memory to hold the image using the fields of info_ptr. */

   /* The easiest way to read the image: */
	{
		row_pointers = malloc(sizeof(png_bytep)*(*rowsP));

		for (i = 0; i < *rowsP; i++)
			row_pointers[i] = malloc(png_get_rowbytes(png_ptr, info_ptr));

		/* Now it's time to read the image.  One of these methods is REQUIRED */
		png_read_image(png_ptr, row_pointers);

		/* read rest of file, and get additional chunks in info_ptr - REQUIRED */
		png_read_end(png_ptr, info_ptr);

		/* clean up after the read, and free any memory allocated - REQUIRED */
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);

#if 0
		for (i=0; i<*rowsP; i++)
		{
			png_bytep	cur_row = row_pointers[i];

			yp = frame[0] + i*width;
			up = u444 + i*width;
			vp = v444 + i*width;

			for (j=0; j<horizontal_size; j++)
			{
				//r=getc(fd); g=getc(fd); b=getc(fd);
				r = *cur_row++; g = *cur_row++; b = *cur_row++;

				/* convert to YUV */
				y = cr*r + cg*g + cb*b;
				u = cu*(b-y);
				v = cv*(r-y);
				yp[j] = (unsigned char)((219.0/256.0)*y + 16.5);  /* nominal range: 16..235 */
				up[j] = (unsigned char)((224.0/256.0)*u + 128.5); /* 16..240 */
				vp[j] = (unsigned char)((224.0/256.0)*v + 128.5); /* 16..240 */
			}
		}
		for (i=0; i<height; i++)
			free(row_pointers[i]);
		free(row_pointers);
#endif
	}
#if 0
	border_extend(frame[0],horizontal_size,vertical_size,width,height);
	border_extend(u444,horizontal_size,vertical_size,width,height);
	border_extend(v444,horizontal_size,vertical_size,width,height);

	if (chroma_format==CHROMA422)
	{
		conv444to422(u444,frame[1]);
		conv444to422(v444,frame[2]);
	}

	if (chroma_format==CHROMA420)
	{
		conv444to422(u444,u422);
		conv444to422(v444,v422);
		conv422to420(u422,frame[1]);
		conv422to420(v422,frame[2]);
	}
#endif
	return((xel**)row_pointers);
}

void png_freearray(xel** rgb_data, unsigned int Fsize_y)
{
	unsigned int i;
	for (i=0; i<Fsize_y; i++)
		free(rgb_data[i]);
	free(rgb_data);
}
